@@ -6,8 +6,8 @@ import torch
 import torchvision.models as models
 from transformers import AutoModel, AutoTokenizer
 
-from jina import Executor, DocumentArray, requests, Document
-from jina.types.arrays.memmap import DocumentArrayMemmap
+from docarray import Document, DocumentArray
+from jina import Executor, requests
 
 
 class Segmenter(Executor):
@@ -16,9 +16,9 @@ class Segmenter(Executor):
         for doc in docs:
             text = doc.tags['caption']
             uri = f'{os.environ["HW_WORKDIR"]}/people-img/{doc.tags["image"]}'
-            chunk_text = Document(text=text, mime_type='text/plain')
-            chunk_uri = Document(uri=uri, mime_type='image/jpeg')
-            doc.chunks = [chunk_text, chunk_uri]
+            chunk_text = Document(text=text)
+            chunk_uri = Document(uri=uri)
+            doc.chunks = DocumentArray([chunk_text, chunk_uri])
             doc.uri = uri
             doc.convert_uri_to_datauri()
 
@@ -61,13 +61,13 @@ class TextEncoder(Executor):
 
     @requests
     def encode(self, docs: 'DocumentArray', **kwargs):
-        with torch.no_grad():
+        with torch.inference_mode():
             if not self.tokenizer.pad_token:
                 self.tokenizer.add_special_tokens({'pad_token': '[PAD]'})
                 self.model.resize_token_embeddings(len(self.tokenizer.vocab))
 
             input_tokens = self.tokenizer(
-                docs.get_attributes('content'),
+                docs[:, 'content'],
                 padding='longest',
                 truncation=True,
                 return_tensors='pt',
@@ -79,8 +79,7 @@ class TextEncoder(Executor):
             outputs = self.model(**input_tokens)
             hidden_states = outputs.hidden_states
 
-            embeds = self._compute_embedding(hidden_states, input_tokens)
-            docs.embeddings = embeds
+            docs.embeddings = self._compute_embedding(hidden_states, input_tokens)
 
 
 class TextCrafter(Executor):
@@ -89,23 +88,19 @@ class TextCrafter(Executor):
 
     @requests()
     def filter(self, docs: DocumentArray, **kwargs):
-        filtered_docs = DocumentArray(
-            d for d in docs.traverse_flat(['c']) if d.mime_type == 'text/plain'
-        )
+        filtered_docs = DocumentArray(d for d in docs['@c'] if d.text != '')
         return filtered_docs
 
 
 class ImageCrafter(Executor):
     @requests(on=['/index', '/search'])
     def craft(self, docs: DocumentArray, **kwargs):
-        filtered_docs = DocumentArray(
-            d for d in docs.traverse_flat(['c']) if d.mime_type == 'image/jpeg'
-        )
+        filtered_docs = DocumentArray(d for d in docs['@c'] if d.text == '')
         target_size = 224
         for doc in filtered_docs:
-            doc.convert_uri_to_image_blob()
-            doc.set_image_blob_shape(shape=(target_size, target_size))
-            doc.set_image_blob_channel_axis(-1, 0)
+            doc.load_uri_to_image_tensor()
+            doc.set_image_tensor_shape(shape=(target_size, target_size))
+            doc.set_image_tensor_channel_axis(-1, 0)
         return filtered_docs
 
 
@@ -137,17 +132,22 @@ class ImageEncoder(Executor):
 
     @requests
     def encode(self, docs: DocumentArray, **kwargs):
-        _input = torch.from_numpy(docs.blobs.astype('float32'))
-        _features = self._get_features(_input).detach()
-        _features = _features.numpy()
-        _features = self._get_pooling(_features)
-        docs.embeddings = _features
+        with torch.inference_mode():
+            _input = torch.from_numpy(docs.tensors.astype('float32'))
+            _features = self._get_features(_input).detach()
+            _features = _features.numpy()
+            _features = self._get_pooling(_features)
+            docs.embeddings = _features
 
 
 class DocVectorIndexer(Executor):
     def __init__(self, index_file_name: str, **kwargs):
         super().__init__(**kwargs)
-        self._docs = DocumentArrayMemmap(self.workspace + f'/{index_file_name}')
+        self._index_file_name = index_file_name
+        if os.path.exists(self.workspace + f'/{index_file_name}'):
+            self._docs = DocumentArray.load(self.workspace + f'/{index_file_name}')
+        else:
+            self._docs = DocumentArray()
 
     @requests(on='/index')
     def index(self, docs: 'DocumentArray', **kwargs):
@@ -162,11 +162,25 @@ class DocVectorIndexer(Executor):
             limit=int(parameters['top_k']),
         )
 
+    @requests(on='/dump')
+    def dump(self, **kwargs):
+        self._docs.save(self.workspace + f'/{self._index_file_name}')
+
+    def close(self):
+        """
+        Stores the DocumentArray to disk
+        """
+        self.dump()
+        super().close()
+
 
 class KeyValueIndexer(Executor):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self._docs = DocumentArrayMemmap(self.workspace + '/kv-idx')
+        if os.path.exists(self.workspace + '/kv-idx'):
+            self._docs = DocumentArray.load(self.workspace + '/kv-idx')
+        else:
+            self._docs = DocumentArray()
 
     @requests(on='/index')
     def index(self, docs: DocumentArray, **kwargs):
@@ -175,9 +189,23 @@ class KeyValueIndexer(Executor):
     @requests(on='/search')
     def query(self, docs: DocumentArray, **kwargs):
         for doc in docs:
+            new_matches = DocumentArray()
             for match in doc.matches:
                 extracted_doc = self._docs[match.parent_id]
-                match.update(extracted_doc)
+                extracted_doc.scores = match.scores
+                new_matches.append(extracted_doc)
+            doc.matches = new_matches
+
+    @requests(on='/dump')
+    def dump(self, **kwargs):
+        self._docs.save(self.workspace + f'/kv-idx')
+
+    def close(self):
+        """
+        Stores the DocumentArray to disk
+        """
+        self.dump()
+        super().close()
 
 
 class WeightedRanker(Executor):
@@ -197,24 +225,26 @@ class WeightedRanker(Executor):
         for d_mod1, d_mod2 in zip(*docs_matrix):
 
             final_matches = {}  # type: Dict[str, Document]
-
             for m in d_mod1.matches:
-                m.scores['relevance'] = m.scores['cosine'].value * d_mod1.weight
+                relevance_score = m.scores['cosine'].value * d_mod1.weight
+                m.scores['relevance'].value = relevance_score
                 final_matches[m.parent_id] = Document(m, copy=True)
 
             for m in d_mod2.matches:
                 if m.parent_id in final_matches:
-                    final_matches[m.parent_id].scores['relevance'] = final_matches[
-                        m.parent_id
-                    ].scores['relevance'].value + (
+                    final_matches[m.parent_id].scores[
+                        'relevance'
+                    ].value = final_matches[m.parent_id].scores['relevance'].value + (
                         m.scores['cosine'].value * d_mod2.weight
                     )
                 else:
-                    m.scores['relevance'] = m.scores['cosine'].value * d_mod2.weight
+                    m.scores['relevance'].value = (
+                        m.scores['cosine'].value * d_mod2.weight
+                    )
                     final_matches[m.parent_id] = Document(m, copy=True)
 
             da = DocumentArray(list(final_matches.values()))
-            da.sort(key=lambda ma: ma.scores['relevance'].value, reverse=True)
+            da = sorted(da, key=lambda ma: ma.scores['relevance'].value, reverse=True)
             d = Document(matches=da[: int(parameters['top_k'])])
             result_da.append(d)
         return result_da
