@@ -1,39 +1,46 @@
 """Module for wrapping Jina Hub API calls."""
 
 import argparse
+import copy
 import hashlib
 import json
 import os
 import random
 from pathlib import Path
-from typing import Optional
-from urllib.parse import urlencode
+from typing import Dict, Optional, Union
 
+from jina import __resources_path__, __version__
+from jina.helper import (
+    ArgNamespace,
+    colored,
+    get_request_header,
+    get_rich_console,
+    retry,
+)
 from jina.hubble import HubExecutor
 from jina.hubble.helper import (
     archive_package,
+    disk_cache_offline,
     download_with_resume,
+    get_cache_db,
+    get_download_cache_dir,
+    get_hubble_error_message,
     get_hubble_url_v2,
     parse_hub_uri,
     upload_file,
-    disk_cache_offline,
 )
 from jina.hubble.hubapi import (
-    install_local,
-    get_dist_path_of_executor,
-    load_secret,
     dump_secret,
+    get_dist_path_of_executor,
     get_lockfile,
+    install_local,
     install_package_dependencies,
+    load_secret,
 )
-from jina import __resources_path__, __version__
-from jina.helper import ArgNamespace, colored, get_request_header
 from jina.importer import ImportExtensions
 from jina.logging.logger import JinaLogger
 from jina.parsers.hubble import set_hub_parser
 
-_cache_file = Path.home().joinpath('.jina', 'disk_cache.db')
-
 
 class HubIO:
     """:class:`HubIO` provides the way to interact with Jina Hub registry.
@@ -59,9 +66,9 @@ class HubIO:
         self.logger = JinaLogger(self.__class__.__name__, **vars(args))
 
         with ImportExtensions(required=True):
-            import rich
             import cryptography
             import filelock
+            import rich
 
             assert rich  #: prevent pycharm auto remove the above line
             assert cryptography
@@ -70,15 +77,14 @@ class HubIO:
     def new(self) -> None:
         """Create a new executor folder interactively."""
 
-        from rich import print, box
-        from rich.prompt import Prompt, Confirm
+        from rich import box, print
         from rich.panel import Panel
-        from rich.table import Table
-        from rich.console import Console
         from rich.progress import track
+        from rich.prompt import Confirm, Prompt
         from rich.syntax import Syntax
+        from rich.table import Table
 
-        console = Console()
+        console = get_rich_console()
 
         print(
             Panel.fit(
@@ -324,13 +330,14 @@ metas:
     def push(self) -> None:
         """Push the executor package to Jina Hub."""
 
-        from rich.console import Console
-
         work_path = Path(self.args.path)
 
         exec_tags = None
+        exec_immutable_tags = None
         if self.args.tag:
             exec_tags = ','.join(self.args.tag)
+        if self.args.protected_tag:
+            exec_immutable_tags = ','.join(self.args.protected_tag)
 
         dockerfile = None
         if self.args.dockerfile:
@@ -344,7 +351,7 @@ metas:
 
             dockerfile = dockerfile.relative_to(work_path)
 
-        console = Console()
+        console = get_rich_console()
         with console.status(f'Pushing `{self.args.path}` ...') as st:
             req_header = get_request_header()
             try:
@@ -367,9 +374,15 @@ metas:
                 if self.args.verbose:
                     form_data['verbose'] = 'True'
 
+                if self.args.no_cache:
+                    form_data['buildWithNoCache'] = 'True'
+
                 if exec_tags:
                     form_data['tags'] = exec_tags
 
+                if exec_immutable_tags:
+                    form_data['immutableTags'] = exec_immutable_tags
+
                 if dockerfile:
                     form_data['dockerfile'] = str(dockerfile)
 
@@ -398,6 +411,7 @@ metas:
                 )
 
                 image = None
+                session_id = req_header.get('jinameta-session-id')
                 for stream_line in resp.iter_lines():
                     stream_msg = json.loads(stream_line)
 
@@ -406,20 +420,22 @@ metas:
                     payload = stream_msg.get('payload', '')
                     if t == 'error':
                         msg = stream_msg.get('message')
-                        if payload and isinstance(payload, dict):
-                            msg = payload.get('message')
-
-                        if 'Process(docker) exited on non-zero code' in msg:
-                            self.logger.error(
-                                '''
-                            Failed on building Docker image. Potential solutions:
-                            - If you haven't provide a Dockerfile in the executor bundle, you may want to provide one,
-                              as the auto-generated one on the cloud did not work.
-                            - If you have provided a Dockerfile, you may want to check the validity of this Dockerfile.
-                            '''
+                        hubble_err = payload
+                        overridden_msg = ''
+                        detail_msg = ''
+                        if isinstance(hubble_err, dict):
+                            (overridden_msg, detail_msg) = get_hubble_error_message(
+                                hubble_err
                             )
+                            if not msg:
+                                msg = detail_msg
 
-                        raise Exception(msg or 'Unknown Error')
+                        if overridden_msg and overridden_msg != detail_msg:
+                            self.logger.warning(overridden_msg)
+
+                        raise Exception(
+                            f'{overridden_msg or msg or "Unknown Error"} session_id: {session_id}'
+                        )
                     if t == 'progress' and subject == 'buildWorkspace':
                         legacy_message = stream_msg.get('legacyMessage', {})
                         status = legacy_message.get('status', '')
@@ -447,23 +463,22 @@ metas:
                     if new_uuid8 != uuid8 or new_secret != secret:
                         dump_secret(work_path, new_uuid8, new_secret)
                 else:
-                    raise Exception('Unknown Error')
+                    raise Exception(f'Unknown Error, session_id: {session_id}')
 
             except KeyboardInterrupt:
                 pass
 
             except Exception as e:  # IO related errors
                 self.logger.error(
-                    f'''Please report this session_id: {colored(req_header["jinameta-session-id"], color="yellow", attrs="bold")} to https://github.com/jina-ai/jina/issues'
-                {e!r}'''
+                    f'''Please report this session_id: [yellow bold]{req_header["jinameta-session-id"]}[/] to https://github.com/jina-ai/jina/issues'''
                 )
                 raise e
 
     def _prettyprint_result(self, console, image):
         # TODO: only support single executor now
 
-        from rich.table import Table
         from rich.panel import Panel
+        from rich.table import Table
 
         uuid8 = image['id']
         secret = image['secret']
@@ -511,57 +526,60 @@ metas:
         flow_plain = f'''from jina import Flow
 
 f = Flow().add(uses='jinahub://{executor_name}')
-
-with f:
-    ...'''
+'''
 
         flow_docker = f'''from jina import Flow
 
 f = Flow().add(uses='jinahub+docker://{executor_name}')
+'''
 
-with f:
-    ...'''
+        flow_sandbox = f'''from jina import Flow
 
-        p1 = Panel(
-            Syntax(
-                flow_plain, 'python', theme='monokai', line_numbers=True, word_wrap=True
-            ),
-            title='Usage',
-            width=80,
-            expand=False,
-        )
-        p2 = Panel(
-            Syntax(
-                flow_docker,
-                'python',
-                theme='monokai',
-                line_numbers=True,
-                word_wrap=True,
-            ),
-            title='Docker usage',
-            width=80,
-            expand=False,
-        )
+f = Flow().add(uses='jinahub+sandbox://{executor_name}')
+'''
+        panels = [
+            Panel(
+                Syntax(
+                    p[0],
+                    'python',
+                    theme='monokai',
+                    word_wrap=True,
+                ),
+                title=p[1],
+                width=80,
+                expand=False,
+            )
+            for p in [
+                (flow_plain, 'Use via source'),
+                (flow_docker, 'Use in Docker'),
+                (flow_sandbox, 'Use in Sandbox'),
+            ]
+        ]
 
         if usage_kind == 'docker':
-            console.print(p2)
+            console.print(panels[1])
         elif usage_kind == 'source':
-            console.print(p1)
+            console.print(panels[0])
         else:
-            console.print(p1, p2)
+            console.print(*reversed(panels))
 
     @staticmethod
-    @disk_cache_offline(cache_file=str(_cache_file))
+    @disk_cache_offline(cache_file=str(get_cache_db()))
     def fetch_meta(
         name: str,
         tag: str,
+        *,
         secret: Optional[str] = None,
+        image_required: bool = True,
+        rebuild_image: bool = True,
         force: bool = False,
     ) -> HubExecutor:
         """Fetch the executor meta info from Jina Hub.
         :param name: the UUID/Name of the executor
         :param tag: the tag of the executor if available, otherwise, use `None` as the value
         :param secret: the access secret of the executor
+        :param image_required: it indicates whether a Docker image is required or not
+        :param rebuild_image: it indicates whether Jina Hub need to rebuild image or not
         :param force: if set to True, access to fetch_meta will always pull latest Executor metas, otherwise, default
             to local cache
         :return: meta of executor
@@ -573,80 +591,93 @@ with f:
         with ImportExtensions(required=True):
             import requests
 
-        pull_url = get_hubble_url_v2() + f'/rpc/executor.getPackage?id={name}'
-        path_params = {}
+        @retry(num_retry=3)
+        def _send_request_with_retry(url, **kwargs):
+            resp = requests.post(url, **kwargs)
+            if resp.status_code != 200:
+                if resp.text:
+                    raise Exception(resp.text)
+                resp.raise_for_status()
+
+            return resp
+
+        pull_url = get_hubble_url_v2() + f'/rpc/executor.getPackage'
+
+        payload = {'id': name, 'include': ['code'], 'rebuildImage': rebuild_image}
+        if image_required:
+            payload['include'].append('docker')
         if secret:
-            path_params['secret'] = secret
+            payload['secret'] = secret
         if tag:
-            path_params['tag'] = tag
-        if path_params:
-            pull_url += f'&{urlencode(path_params)}'
+            payload['tag'] = tag
 
-        resp = requests.get(pull_url, headers=get_request_header())
-        if resp.status_code != 200:
-            if resp.text:
-                raise Exception(resp.text)
-            resp.raise_for_status()
+        req_header = get_request_header()
 
+        resp = _send_request_with_retry(pull_url, json=payload, headers=req_header)
         resp = resp.json()['data']
 
+        images = resp['package'].get('containers', [])
+        image_name = images[0] if images else None
+        if image_required and not image_name:
+            raise Exception(
+                f'No image found for executor "{name}", '
+                f'tag: {tag}, commit: {resp.get("commit", {}).get("id")}, '
+                f'session_id: {req_header.get("jinameta-session-id")}'
+            )
+
         return HubExecutor(
             uuid=resp['id'],
             name=resp.get('name', None),
-            sn=resp.get('sn', None),
+            commit_id=resp['commit'].get('id'),
             tag=tag or resp['commit'].get('tags', [None])[0],
             visibility=resp['visibility'],
-            image_name=resp['package'].get('containers', [None])[0],
+            image_name=image_name,
             archive_url=resp['package']['download'],
             md5sum=resp['package']['md5'],
         )
 
     @staticmethod
-    def deploy_public_sandbox(uses: str):
+    def deploy_public_sandbox(args: Union[argparse.Namespace, Dict]) -> str:
         """
         Deploy a public sandbox to Jina Hub.
-        :param uses: the executor uses string
+        :param args: arguments parsed from the CLI
 
         :return: the host and port of the sandbox
         """
-        scheme, name, tag, secret = parse_hub_uri(uses)
+        args_copy = copy.deepcopy(args)
+        if not isinstance(args_copy, Dict):
+            args_copy = vars(args_copy)
+
+        scheme, name, tag, secret = parse_hub_uri(args_copy.pop('uses', ''))
         payload = {
             'name': name,
             'tag': tag if tag else 'latest',
             'jina': __version__,
+            'args': args_copy,
         }
 
-        from rich.progress import Console
         import requests
 
-        console = Console()
+        console = get_rich_console()
 
         host = None
         port = None
-        try:
-            json_response = requests.get(
-                url=get_hubble_url_v2() + '/rpc/sandbox.get',
-                params=payload,
-                headers=get_request_header(),
-            ).json()
-            if json_response.get('code') == 200:
-                host = json_response.get('data', {}).get('host', None)
-                port = json_response.get('data', {}).get('port', None)
-                livetime = json_response.get('data', {}).get('livetime', '15 mins')
 
-        except Exception:
-            raise
+        json_response = requests.post(
+            url=get_hubble_url_v2() + '/rpc/sandbox.get',
+            json=payload,
+            headers=get_request_header(),
+        ).json()
+        if json_response.get('code') == 200:
+            host = json_response.get('data', {}).get('host', None)
+            port = json_response.get('data', {}).get('port', None)
 
         if host and port:
-            console.log(f"There is already one sandbox existing, reusing it.")
-            console.log(
-                f"[bold green]This sandbox will be removed when no traffic during {livetime}"
-            )
-
+            console.log(f"🎉 A sandbox already exists, reusing it.")
             return host, port
 
         with console.status(
-            f"[bold green]Deploying sandbox for ({name}) since no existing one..."
+            f"[bold green]🚧 Deploying sandbox for [bold white]{name}[/bold white] since none exists..."
         ):
             try:
                 json_response = requests.post(
@@ -655,24 +686,23 @@ with f:
                     headers=get_request_header(),
                 ).json()
 
-                host = json_response.get('data', {}).get('host', None)
-                port = json_response.get('data', {}).get('port', None)
-                livetime = json_response.get('data', {}).get('livetime', '15 mins')
+                data = json_response.get('data') or {}
+                host = data.get('host', None)
+                port = data.get('port', None)
                 if not host or not port:
                     raise Exception(f'Failed to deploy sandbox: {json_response}')
 
-                console.log(f"Deployment completed: {host}:{port}")
-                console.log(
-                    f"[bold green]This sandbox will be removed when no traffic during {livetime}"
-                )
+                console.log(f"🎉 Deployment completed, using it.")
             except:
-                console.log("Deployment failed")
+                console.log(
+                    "🚨 Deployment failed, feel free to raise an issue. https://github.com/jina-ai/jina/issues/new"
+                )
                 raise
 
         return host, port
 
     def _pull_with_progress(self, log_streams, console):
-        from rich.progress import Progress, DownloadColumn, BarColumn
+        from rich.progress import BarColumn, DownloadColumn, Progress
 
         with Progress(
             "[progress.description]{task.description}",
@@ -712,7 +742,6 @@ with f:
     def _load_docker_client(self):
         with ImportExtensions(required=True):
             import docker.errors
-            import docker
             from docker import APIClient
 
             from jina import __windows__
@@ -737,9 +766,7 @@ with f:
         :return: the `uses` string
         """
 
-        from rich.console import Console
-
-        console = Console()
+        console = get_rich_console()
         cached_zip_file = None
         executor_name = None
         usage_kind = None
@@ -748,10 +775,15 @@ with f:
             need_pull = self.args.force_update
             with console.status(f'Pulling {self.args.uri}...') as st:
                 scheme, name, tag, secret = parse_hub_uri(self.args.uri)
+                image_required = scheme == 'jinahub+docker'
 
                 st.update(f'Fetching [bold]{name}[/bold] from Jina Hub ...')
                 executor, from_cache = HubIO.fetch_meta(
-                    name, tag, secret=secret, force=need_pull
+                    name,
+                    tag,
+                    secret=secret,
+                    image_required=image_required,
+                    force=need_pull,
                 )
 
                 presented_id = getattr(executor, 'name', executor.uuid)
@@ -790,10 +822,12 @@ with f:
                             pkg_path, pkg_dist_path = get_dist_path_of_executor(
                                 executor
                             )
-                            # check serial number to upgrade
-                            sn_file_path = pkg_dist_path / f'PKG-SN-{executor.sn or 0}'
-                            if (not sn_file_path.exists()) and any(
-                                pkg_dist_path.glob('PKG-SN-*')
+                            # check commit id to upgrade
+                            commit_file_path = (
+                                pkg_dist_path / f'PKG-COMMIT-{executor.commit_id or 0}'
+                            )
+                            if (not commit_file_path.exists()) and any(
+                                pkg_dist_path.glob('PKG-COMMIT-*')
                             ):
                                 raise FileNotFoundError(
                                     f'{pkg_path} need to be upgraded'
@@ -813,21 +847,17 @@ with f:
                             # pull the latest executor meta, as the cached data would expire
                             if from_cache:
                                 executor, _ = HubIO.fetch_meta(
-                                    name, tag, secret=secret, force=True
+                                    name,
+                                    tag,
+                                    secret=secret,
+                                    image_required=False,
+                                    force=True,
                                 )
 
-                            cache_dir = Path(
-                                os.environ.get(
-                                    'JINA_HUB_CACHE_DIR',
-                                    Path.home().joinpath('.cache', 'jina'),
-                                )
-                            )
-                            cache_dir.mkdir(parents=True, exist_ok=True)
-
                             st.update(f'Downloading {name} ...')
                             cached_zip_file = download_with_resume(
                                 executor.archive_url,
-                                cache_dir,
+                                get_download_cache_dir(),
                                 f'{executor.uuid}-{executor.md5sum}.zip',
                                 md5sum=executor.md5sum,
                             )
