@@ -5,6 +5,7 @@ from datetime import datetime
 from typing import Dict, List, Optional, Tuple
 
 from jina.serve.networking import GrpcConnectionPool
+from jina.serve.runtimes.request_handlers.data_request_handler import DataRequestHandler
 from jina.types.request.data import DataRequest
 
 
@@ -18,6 +19,7 @@ class TopologyGraph:
         will be considered to be hanging, and they will be "flagged" so that the user can ignore their tasks and not await them.
 
     :param conditions: A dictionary describing which Executors have special conditions to be fullfilled by the `Documents` to be sent to them.
+    :param reduce: Reduce requests arriving from multiple needed predecessors, True by default
     """
 
     class _ReqReplyNode:
@@ -27,6 +29,8 @@ class TopologyGraph:
             number_of_parts: int = 1,
             hanging: bool = False,
             filter_condition: dict = None,
+            reduce: bool = True,
+            timeout_send: Optional[float] = None,
         ):
             self.name = name
             self.outgoing_nodes = []
@@ -37,6 +41,8 @@ class TopologyGraph:
             self.end_time = None
             self.status = None
             self._filter_condition = filter_condition
+            self._reduce = reduce
+            self._timeout_send = timeout_send
 
         @property
         def leaf(self):
@@ -61,7 +67,7 @@ class TopologyGraph:
             if previous_task is not None:
                 result = await previous_task
                 request, metadata = result[0], result[1]
-            if 'is-error' in metadata:
+            if metadata and 'is-error' in metadata:
                 return request, metadata
             elif request is not None:
                 self.parts_to_send.append(request)
@@ -70,15 +76,20 @@ class TopologyGraph:
                     self.start_time = datetime.utcnow()
                     if self._filter_condition is not None:
                         self._update_requests()
+                    if self._reduce and len(self.parts_to_send) > 1:
+                        self.parts_to_send = [
+                            DataRequestHandler.reduce_requests(self.parts_to_send)
+                        ]
 
                     resp, metadata = await connection_pool.send_requests_once(
                         requests=self.parts_to_send,
                         deployment=self.name,
                         head=True,
                         endpoint=endpoint,
+                        timeout=self._timeout_send,
                     )
                     self.end_time = datetime.utcnow()
-                    if 'is-error' in metadata:
+                    if metadata and 'is-error' in metadata:
                         self.status = resp.header.status
                     return resp, metadata
 
@@ -171,7 +182,13 @@ class TopologyGraph:
             return request
 
     def __init__(
-        self, graph_representation: Dict, graph_conditions: Dict = {}, *args, **kwargs
+        self,
+        graph_representation: Dict,
+        graph_conditions: Dict = {},
+        deployments_disable_reduce: List[str] = [],
+        timeout_send: Optional[float] = 1.0,
+        *args,
+        **kwargs
     ):
         num_parts_per_node = defaultdict(int)
         if 'start-gateway' in graph_representation:
@@ -200,6 +217,8 @@ class TopologyGraph:
                 else 1,
                 hanging=node_name in hanging_deployment_names,
                 filter_condition=condition,
+                reduce=node_name not in deployments_disable_reduce,
+                timeout_send=timeout_send,
             )
 
         for node_name, outgoing_node_names in graph_representation.items():
