@@ -3,7 +3,8 @@ from typing import TYPE_CHECKING, Dict, List, Optional
 from docarray import DocumentArray
 
 from jina import __default_endpoint__
-from jina.excepts import BadConfigSource, ExecutorFailToLoad
+from jina.excepts import BadConfigSource
+from jina.importer import ImportExtensions
 from jina.serve.executors import BaseExecutor
 from jina.types.request.data import DataRequest
 
@@ -38,6 +39,27 @@ class DataRequestHandler:
         self.logger = logger
         self._is_closed = False
         self._load_executor(metrics_registry)
+        self._init_monitoring(metrics_registry)
+
+    def _init_monitoring(self, metrics_registry: Optional['CollectorRegistry'] = None):
+
+        if metrics_registry:
+
+            with ImportExtensions(
+                required=True,
+                help_text='You need to install the `prometheus_client` to use the montitoring functionality of jina',
+            ):
+                from prometheus_client import Counter
+
+                self._counter = Counter(
+                    'document_processed',
+                    'Number of Documents that have been processed by the executor',
+                    namespace='jina',
+                    labelnames=('endpoint', 'executor'),
+                    registry=metrics_registry,
+                )
+        else:
+            self._counter = None
 
     def _load_executor(self, metrics_registry: Optional['CollectorRegistry'] = None):
         """
@@ -62,18 +84,18 @@ class DataRequestHandler:
                 extra_search_paths=self.args.extra_search_paths,
             )
 
-        except BadConfigSource as ex:
+        except BadConfigSource:
             self.logger.error(
                 f'fail to load config from {self.args.uses}, if you are using docker image for --uses, '
                 f'please use "docker://YOUR_IMAGE_NAME"'
             )
-            raise ExecutorFailToLoad from ex
-        except FileNotFoundError as ex:
+            raise
+        except FileNotFoundError:
             self.logger.error(f'fail to load file dependency')
-            raise ExecutorFailToLoad from ex
-        except Exception as ex:
+            raise
+        except Exception:
             self.logger.critical(f'can not load the executor from {self.args.uses}')
-            raise ExecutorFailToLoad from ex
+            raise
 
     @staticmethod
     def _parse_params(parameters: Dict, executor_name: str):
@@ -135,6 +157,11 @@ class DataRequestHandler:
                     f'but getting {return_data!r}'
                 )
 
+        if self._counter:
+            self._counter.labels(
+                requests[0].header.exec_endpoint, self._executor.__class__.__name__
+            ).inc(len(docs))
+
         DataRequestHandler.replace_docs(requests[0], docs, self.args.output_array_type)
 
         return requests[0]
